<?php 
require_once(LIB_PATH.DS.'fax-email-south-africa-requirements.php');

class Service
{
	protected static $table_name="services";
	protected static $db_fields = array('id', 'service', 'pagename', 'position');
	public $id;
	public $service;
	public $pagename;
	public $position;
	
	public $orig_position;
	public $orig_pagename;
	
	
	// Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY position ASC");
	}
	
	public static function find_en()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE af=0 ORDER BY position ASC");
	}
	
	public static function find_af()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE af=1 ORDER BY position ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		// More dynamic, short-form approach:
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		//get_object_vars returns an associative array with all attributes 
		//(incl. private ones!!) as the keys and their current values as the value
		$object_vars = $this->attributes();
		// We don't care about the value, we just want to know if the key exists
		// Will return true or false
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes; // for more complex tables can use SQL show_fields_from (list) can be parsed then build array dynamically
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	public function save()
	{
		if (isset($this->id))
		{
			$this->update();
		} else
		{
			$this->create();
		}
	}
	
	public function create()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position+1 WHERE position>={$this->position}";
		$database->query($query);
		
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			$this->add_rewrite_rule();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;

		if ($this->orig_position<$this->position)
		{
			$this->fill_gaps();
		} elseif ($this->orig_position>$this->position)
		{
			$this->check_double();
		}
		
		if($this->pagename != $this->orig_pagename)
			$this->edit_rewrite_rule();
	//	if($this->service_af != $this->orig_service_af)
	//		$this->edit_rewrite_rule_af();

		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return ($database->affected_rows() >0) ? true: false;
	}
	
	public function check_double()
	{				
		global $database;

		$query = "SELECT * FROM ".self::$table_name;
		$result = $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] < $this->orig_position && $row['position'] >= $this->position)
			{
				$update = "UPDATE ".self::$table_name." SET position=position+1 WHERE id={$row['id']}";
				$database->query($update);
			}
		}
		return true;
	}
	
	public function fill_gaps()
	{				
		global $database;

		$query  = "SELECT * FROM ".self::$table_name;
		$result	= $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] > $this->orig_position && $row['position'] <= $this->position)
			{
				$shift = "UPDATE ".self::$table_name." SET position=position-1 WHERE id='{$row['id']}'";
				$database->query($shift);
			}
		}
		return true;
	}
	
	public function delete()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position-1 WHERE position>{$this->position}";
		$database->query($query);
		
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		if($database->query($sql))
		{
			$this->remove_rewrite_rule();
		}
		return ($database->affected_rows() >= 1) ? true : false;
	}
	
	protected function add_rewrite_rule()
	{
		global $database;
		$file = SITE_ROOT.DS.'.htaccess';
		
	//	$page = Page::find_by_id(3);
	//	$template = Template::find_by_id(2);
		
		$content="";
	 	$text = "";
		
	  	$text .= "RewriteRule ^";
	  	$text .= str_replace(".html","\.html",$this->pagename);
//	  	$text .= strtolower(str_replace(' ', '-', str_replace($template->keyword, $this->service, str_replace(".html","\.html",$page->pagename))));
	  	$text .= "$ fax-service.php?id=";
	  	$text .= $this->id;
	  	$text .= "\n";
//	  	$text .= "RewriteRule ^";
//	  	$text .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->service_af)), str_replace(".html","\.html",$page->pagename_af));
//	  	$text .= "$ faks-dienst.php?id=";
//	  	$text .= $this->id;
//	  	$text .= "\n";
	  	
	  	$string = file($file);
	  	$count = count($string);
		for($i=0;$i<$count-1;$i++)
		{
			$content .= $string[$i];
		}
		$content .= $text;
		$content .= 'RewriteRule ^(.*)\.html(.*)$ $1\.php$2';
	
		if ($handle = fopen($file,'w+'))
		{	
			fwrite($handle, $content);
			fclose($handle);
			return true;
		} else 
		{ 
			global $session;
			$session->message = "Could not open .htaccess file for writing.";
			return false;
		}
 	}
 	
	protected function edit_rewrite_rule()
	{
		$file = SITE_ROOT.DS.'.htaccess';
		
	//	$page = Page::find_by_id(3);
	//	$template = Template::find_by_id(2);
		
		$old  = "RewriteRule ^";
		$old .= str_replace(".html","\.html",$this->orig_pagename);
	//  $old .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->orig_service)), str_replace(".html","\.html",$page->pagename));
	  	$old .= "$ fax-service.php?id=";
	  	$old .= $this->id;
	  	
	  	$new  = "RewriteRule ^";
		$new .= str_replace(".html","\.html",$this->pagename);
	//  $new .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->service)), str_replace(".html","\.html",$page->pagename));
	  	$new .= "$ fax-service.php?id=";
	  	$new .= $this->id;
	  	$new .= "\n";
	  			
		$string = file($file);
		$content ="";
		for($i = 0; $i < count($string); $i++)
		{
			if(strstr($string[$i],$old))
			{
				$content .= $new;
			} 
			else
			{
				$content .= $string[$i];
			}
		}
	 	
		if ($handle = fopen($file,'w'))
		{
			if (fwrite($handle,$content))
			{
				fclose($handle);
				return true;
			}
		} else 
		{
			global $session;
			$session->message ="Error opening file.";
		}
 	}	
 	
	protected function edit_rewrite_rule_af()
	{
		$file = SITE_ROOT.DS.'.htaccess';
		
		$page = Page::find_by_id(3);
		$template = Template::find_by_id(2);
		
		$old = "RewriteRule ^";
	  	$old .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->orig_service_af)), str_replace(".html","\.html",$page->pagename_af));
	  	$old .= "$ faks-service.php?id=";
	  	$old .= $this->id;
	  	
	  	$new = "RewriteRule ^";
	  	$new .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->service_af)), str_replace(".html","\.html",$page->pagename_af));
	  	$new .= "$ faks-dienst.php?id=";
	  	$new .= $this->id;
	  	$new .= "\n";
	  			
		$string = file($file);
		$content ="";
		for($i = 0; $i < count($string); $i++)
		{
			if(strstr($string[$i],$old))
			{
				$content .= $new;
			} 
			else
			{
				$content .= $string[$i];
			}
		}
	 	
		if ($handle = fopen($file,'w'))
		{
			if (fwrite($handle,$content))
			{
				fclose($handle);
				return true;
			}
		} else 
		{
			global $session;
			$session->message ="Error opening file.";
		}
 	}	
 	
	protected function remove_rewrite_rule()
	{
		$file = SITE_ROOT.DS.'.htaccess';
		
		$old_en  = "RewriteRule ^";
		$old_en .= str_replace(".html","\.html",$this->pagename);
	  	$old_en .= "$ fax-service.php?id=";
	  	$old_en .= $this->id;
	  	
//	  	$old_af = "RewriteRule ^";
//	  	$old_af .= str_replace($template->keyword, strtolower(str_replace(' ', '-', $this->service_af)), str_replace(".html","\.html",$page->pagename_af));
//	  	$old_af .= "$ faks-dienst.php?id=";
//	  	$old_af .= $this->id;
	  			
		$string = file($file);
		$content ="";
		for($i = 0; $i < count($string); $i++)
		{
		//	if(strstr($string[$i],$old_en) || strstr($string[$i],$old_af))
			if(strstr($string[$i],$old_en))
			{
				$content .= "";
			} 
			else
			{
				$content .= $string[$i];
			}
		}
	 	
		if ($handle = fopen($file,'w'))
		{
			if (fwrite($handle,$content))
			{
				fclose($handle);
				return true;
			}
		} else 
		{
			global $session;
			$session->message ="Error opening file.";
		}
 	}
}

?>