<?php 
require_once(LIB_PATH.DS.'gitaren-nederland-database.php');

class ProductImage
{
	protected static $table_name="product_images";
	protected static $db_fields = array('id', 'product_id', 'filename', 'position');
	public $id;
	public $product_id;
	public $filename;
	public $position;
	
	public $orig_position;
	
	private $temp_path;
	protected $upload_dir="gitaarwinkel-zuid-nederland-produkten";
	public $errors=array();
	
	// Common Database Methods	
	public static function find_all()
	{
		return self::find_by_sql("SELECT * FROM ".self::$table_name." ORDER BY position ASC");
	}
	
	public static function find_by_id($id=0)
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE id={$id} LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}
	
	public static function find_by_product($product)
	{
		global $database;
		return self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE product_id={$product} ORDER BY position ASC");
	}
	
	public static function find_first_for_product($product="")
	{
		global $database;
		$result_array = self::find_by_sql("SELECT * FROM ".self::$table_name." WHERE product_id={$product} AND position=1 LIMIT 1");
		return !empty($result_array) ? array_shift($result_array) : false;
	}	
	
	public static function find_by_sql($sql="")
	{
		global $database;
		$result_set = $database->query($sql);
		$object_array = array();
		while ($row = $database->fetch_array($result_set))
		{
			$object_array[] = self::instantiate($row);
		}
		return $object_array;
	
	}
	
	private static function instantiate($record)
	{
		// could check that $record exists and is an array
		$object = new self;
		
		// More dynamic, short-form approach:
		foreach($record as $attribute=>$value)
		{
			if($object->has_attribute($attribute))
			{
				$object->$attribute = $value;
			}
		}
		return $object;		
	}
	
	private function has_attribute($attribute)
	{
		//get_object_vars returns an associative array with all attributes 
		//(incl. private ones!!) as the keys and their current values as the value
		$object_vars = $this->attributes();
		// We don't care about the value, we just want to know if the key exists
		// Will return true or false
		return array_key_exists($attribute, $object_vars);
	}

	protected function attributes()
	{  	// return an array of attribute keys and their values
		$attributes = array();
		foreach(self::$db_fields as $field)
		{
			if(property_exists($this, $field))
			{
				$attributes[$field] = $this->$field; // use dynamic $field
			}
		}
		return $attributes; // for more complex tables can use SQL show_fields_from (list) can be parsed then build array dynamically
	}
	
	protected function sanitized_attributes()
	{
		global $database;
		$clean_attributes = array();
		foreach($this->attributes() as $key => $value)
		{
			$clean_attributes[$key] = $database->escape_value($value);
		}
		return $clean_attributes;
	}
	
	protected $upload_errors = array
	( 		// http://www.php.net/manual/en/features.file-upload.errors.php
		UPLOAD_ERR_OK			=> "Geen fouten.",
		UPLOAD_ERR_INI_SIZE		=> "Groter dan max. upload bestandsgrootte.",
		UPLOAD_ERR_FORM_SIZE	=> "Groter dan de max. bestandsgrootte van vorm.",
		UPLOAD_ERR_PARTIAL		=> "Gedeeltelijke upload.",
		UPLOAD_ERR_NO_FILE		=> "Geen bestand ge-upload.",
		UPLOAD_ERR_NO_TMP_DIR	=> "Geen tijdelijke map.",
		UPLOAD_ERR_CANT_WRITE	=> "Kan niet opslaan.",
		UPLOAD_ERR_EXTENSION 	=> "Bestandstype niet toegestaan.",
	);
	
	// Pass in $_FILE(['uploaded_file']) as an argument
	public function attach_file($file)
	{
		// Perform error checking on the form parameters
		if(!$file || empty($file) || !is_array($file))
		{	// error: nothing uploaded or wrong argument usage
			$this->errors[] = "Er is geen bestand uploaded.";
			return false;
		} elseif($file['error'] !=0)
		{	// error: report what PHP says went wrong
			$this->errors[] = $this->upload_errors[$file['error']];
			return false;
		} else
		{
			// Set object attributes to the form parameters.
			$this->temp_path 	= $file['tmp_name'];			
			$this->filename		= str_replace(" ","-",basename($file['name']));
			// Don't worry about saving anything to database yet				
			return true;
		}
	}
	
	public function image_path()
	{
		return $this->upload_dir.DS.$this->filename;
	}
	
	public function image_display()
	{
		return $this->upload_dir.'/'.$this->filename;
	}
	
	public function thumb_path()
	{
		$template = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
		$image_info = getimagesize($template);
		$image_type = $image_info[2];
		if ( $image_type == 1 ) 
		{
			$thumb_path = $this->upload_dir.DS.'thumbs'.DS.str_replace('.gif','.jpg',strtolower($this->filename));
		} else
		{
			$thumb_path = $this->upload_dir.DS.'thumbs'.DS.$this->filename;
		}
		return $thumb_path;
	}
	
	public function thumb_display()
	{
		$template = SITE_ROOT.'/'.$this->image_path();
		$image_info = getimagesize($template);
		$image_type = $image_info[2];
		
		if ( $image_type == 1 ) 
		{
			$thumb_path = $this->upload_dir.'/thumbs/'.str_replace('.gif','.jpg',strtolower($this->filename));
		} else
		{
			$thumb_path = $this->upload_dir.'/thumbs/'.$this->filename;
		} 
		return $thumb_path;
	}
	
	public function small_path()
	{
		$template = SITE_ROOT.DS.$this->image_path();
		$image_info = getimagesize($template);
		$image_type = $image_info[2];
		
		if( $image_type == 1 ) 
		{
			$small_path = $this->upload_dir.DS.'small'.DS.str_replace('.gif','.jpg',strtolower($this->filename));
		}  else
		{
			$small_path = $this->upload_dir.DS.'small'.DS.$this->filename;
		}
		return $small_path;
	}
	
	public function small_display()
	{
		$template = SITE_ROOT.'/'.$this->image_path();
		$image_info = getimagesize($template);
		$image_type = $image_info[2];
		
		if( $image_type == 1 ) 
		{
			$small_path = $this->upload_dir.'/small/'.str_replace('.gif','.jpg',strtolower($this->filename));
		} else
		{
			$small_path = $this->upload_dir.'/small/'.$this->filename;
		} 
		return $small_path;
	}
	
	public function save()
	{	
		if (isset($this->id))
		{
			$this->update();
			return true;
		} else
		{		
			// Make sure there are no errors. 
			// Can't save if there are pre-existing errors
			if(!empty($this->errors)) {return false;}
			
			// Can't save without filename and temp location
			if(empty($this->filename) || empty($this->temp_path))
			{
				$this->errors[] = "De locatie van de bestanden was niet beschikbaar.";
				return false;
			}
			
			// Determine the target_path
			$target_path = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
			
			// Make sure file doesn't already exist in the target location
			if(file_exists($target_path))
			{
				$this->errors[] = "Het bestand {$this->filename} bestaat al.";
				return false;
			}
			
			// Attempt to move the file
			if(move_uploaded_file($this->temp_path, $target_path))
			{	
				// Resize images for product page
				$this->resize_image();
				$this->create_thumbnail();
				$this->create_small();
				
				// Save a corresponding entry to database
				if($this->create())
				{
					unset($this->temp_path);
					return true;
				}
			} else
			{	// File was not moved
				$this->errors[] = "Het uploaden is mislukt, mogenlijk als gevolg van onjuiste machtigingen voor de upload map.";
				return false;
			}
			
			// Save a corresponding entry to the database
			$this->create();
		}
	
	}
	
	public function create()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position+1 WHERE product_id={$this->product_id} AND position>={$this->position}";
		$database->query($query);
		
		$attributes = $this->sanitized_attributes();
		
		$sql  = "INSERT INTO ".self::$table_name." (";
		$sql .= join(", ", array_keys($attributes));
		$sql .= ") VALUES ('";
		$sql .= join("', '", array_values($attributes));
		$sql .= "')";
		
		if($database->query($sql))
		{
			$this->id = $database->insert_id();
			return true;
		} else
		{
			return false;
		}
	}
	
	public function update()
	{
		global $database;
		
		if(isset($this->orig_position))
		{
			if ($this->orig_position<$this->position)
			{
				$this->fill_gaps();
			} elseif ($this->orig_position>$this->position)
			{
				$this->check_double();
			}	
		}

		$attributes = $this->sanitized_attributes();
		$attribute_pairs = array();
		foreach ($attributes as $key => $value)
		{
			$attribute_pairs[] = "{$key}='{$value}'";
		}
		
		$sql  = "UPDATE ".self::$table_name." SET ";
		$sql .= join(", ", $attribute_pairs);
		$sql .= " WHERE id=".$database->escape_value($this->id); //dont have to escape id, but no harm
		$database->query($sql);
		return ($database->affected_rows() >0) ? true: false;
	}
	
	public function delete()
	{
		global $database;
		
		$query = "UPDATE ".self::$table_name." SET position=position-1 WHERE product_id={$this->product_id} AND position>{$this->position}";
		$database->query($query);
		
		$sql  = "DELETE FROM ".self::$table_name;
		$sql .= " WHERE id=".$database->escape_value($this->id);
		$sql .= " LIMIT 1";
		$database->query($sql);
		return ($database->affected_rows() == 1) ? true : false;
	}
	
	public function destroy()
	{	// First remove db entry
		if($this->delete())
		{
			// then remove the file
			// Note that even though the database entry is gone, 
			// this object is still around (which lets us use $this->image_path()).
			$thumb_path = SITE_ROOT.DS.$this->thumb_path();
			if(file_exists($thumb_path))
			{
				unlink($thumb_path);
			}	
			
			$small_path = SITE_ROOT.DS.$this->small_path();
			if(file_exists($small_path))
			{
				unlink($small_path);
			}
		
			$target_path = SITE_ROOT.DS.$this->image_path();
			if (unlink($target_path))
			{
				return true;
			}
		} else
		{
			// database delete failed
			return false;
		}
	}
	
	public function check_double()
	{				
		global $database;

		$query = "SELECT * FROM ".self::$table_name." WHERE product_id={$this->product_id}";
		$result = $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] < $this->orig_position && $row['position'] >= $this->position)
			{
				$update = "UPDATE ".self::$table_name." SET position=position+1 WHERE id={$row['id']}";
				$database->query($update);
			}
		}
		return true;
	}
	
	public function fill_gaps()
	{				
		global $database;

		$query  = "SELECT * FROM ".self::$table_name." WHERE product_id={$this->product_id}";
		$result	= $database->query($query);
		while ($row = mysql_fetch_array($result))
		{
			if ($row['position'] > $this->orig_position && $row['position'] <= $this->position)
			{
				$shift = "UPDATE ".self::$table_name." SET position=position-1 WHERE id='{$row['id']}'";
				$database->query($shift);
			}
		}
			
			return true;
	}
	
	public function create_small()
	{
		$template = SITE_ROOT.DS.$this->image_path();
		$thumb_width = 80;
		
		$image_info = getimagesize($template);
		$width = $image_info[0];
		$height = $image_info[1];
		$image_type = $image_info[2];
		
		$ratio = $thumb_width/$width;
		$thumb_height = round($height*$ratio);
		
		$resized_image = imagecreatetruecolor($thumb_width, $thumb_height);
		
		if( $image_type == 3 ) 
		{
			$image = imagecreatefrompng($template);
			$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'small'.DS.$this->filename;
			
			//Resize
			imagecopyresampled($resized_image, $image, 0, 0, 0, 0, $thumb_width, $thumb_height, $width, $height);
			$resized_image;
			
			//output
			header('Content-type: image/png');
			imagepng($resized_image ,$thumb_path, 70); 
			imagedestroy($resized_image);
		} else
		{
			if( $image_type == 2 ) 
			{
				$image = imagecreatefromjpeg($template);
				$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'small'.DS.$this->filename;
			} elseif( $image_type == 1 ) 
			{
				$image = imagecreatefromgif($template);
				$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'small'.DS.str_replace('.gif','.jpg',strtolower($this->filename));
			} 
			
			//Resize
			imagecopyresampled($resized_image, $image, 0, 0, 0, 0, $thumb_width, $thumb_height, $width, $height);
			$resized_image;
			
			//output
			header('Content-type: image/jpeg');
			imagejpeg($resized_image ,$thumb_path, 70); 
			imagedestroy($resized_image);
		}
		return true;
	}
	
	public function create_thumbnail()
	{
		$template = SITE_ROOT.DS.$this->image_path();
		$thumb_size = 120;
		
		// Get info
		$image_info = getimagesize($template);
		$width = $image_info[0];
		$height = $image_info[1];
		$image_type = $image_info[2];
		
		// Set new info
		if($width>$height)
		{
			$ratio = $thumb_size/$height;
			$resize_height = $thumb_size;
			$resize_width = round($width*$ratio);
			$y = 0;
			$x = -($resize_width-$thumb_size)/2;
		} else
		{
			$ratio = $thumb_size/$width;
			$resize_width = $thumb_size;
			$resize_height = round($height*$ratio);
			$x = 0;
			$y = -($resize_height-$thumb_size)/2;
		}

		if( $image_type == 3 ) 
		{
			$image = imagecreatefrompng($template);
			$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'thumbs'.DS.str_replace('.png','.jpg',strtolower($this->filename));
			
			//Resize
			$resized_image = imagecreatetruecolor($thumb_size, $thumb_size);
			imagecopyresampled($resized_image, $image, $x, $y, 0, 0, $resize_width, $resize_height, $width, $height);
			$thumb = $resized_image;
			
			//output
			header('Content-type: image/png');
			imagejpeg($thumb ,$thumb_path, 80); 
			imagedestroy($thumb);
		} else
		{
			if( $image_type == 2 ) 
			{
				$image = imagecreatefromjpeg($template);
				$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'thumbs'.DS.$this->filename;
			} elseif( $image_type == 1 ) 
			{
				$image = imagecreatefromgif($template);
				$thumb_path = SITE_ROOT.DS.$this->upload_dir.DS.'thumbs'.DS.str_replace('.gif','.jpg',strtolower($this->filename));
			}
			
			//Resize
			$resized_image = imagecreatetruecolor($thumb_size, $thumb_size);
			imagecopyresampled($resized_image, $image, $x, $y, 0, 0, $resize_width, $resize_height, $width, $height);
			$thumb = $resized_image;
			
			//output
			header('Content-type: image/jpeg');
			imagejpeg($thumb ,$thumb_path, 80); 
			imagedestroy($thumb);
		}
		return true;
	}
	
	public function resize_image()
	{
		$template = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
		
		// Get info
		$image_info = getimagesize($template);
		$height = $image_info[1];
		
		if($height > 450)
		{
			$width = $image_info[0];
			$image_type = $image_info[2];
			$resize_height = 450;
			
			$ratio = $resize_height/$height;
			$resize_width = round($width*$ratio);
			
			$resized_image = imagecreatetruecolor($resize_width, $resize_height);
			
			if($image_type == 3)
			{
				$image = imagecreatefrompng($template);
				$image_path = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
				
				imagecopyresampled($resized_image, $image, 0, 0, 0, 0, $resize_width, $resize_height, $width, $height);
				$resized_image;
				
				//output
				header('Content-type: image/png');
				imagepng($resized_image ,$image_path, 70); 
				imagedestroy($resized_image);
			} else
			{
				if( $image_type == 2 ) 
				{
					$image = imagecreatefromjpeg($template);
					$image_path = SITE_ROOT.DS.$this->upload_dir.DS.$this->filename;
				}  elseif( $image_type == 1 ) 
				{
					$image = imagecreatefromgif($template);
					$image_path = SITE_ROOT.DS.$this->upload_dir.DS.str_replace('.gif','.jpg',strtolower($this->filename));
					$this->filename = str_replace('.gif','.jpg',strtolower($this->filename));
				} 
				
				imagecopyresampled($resized_image, $image, 0, 0, 0, 0, $resize_width, $resize_height, $width, $height);
				$resized_image;
				
				//output
				header('Content-type: image/jpeg');
				imagejpeg($resized_image ,$image_path, 70); 
				imagedestroy($resized_image);
			}
		}
		return true;
	}
}

?>